==> kenga-0.0.51.tgz: package/bound.js <==
'use strict';

var _classCallCheck = require('@babel/runtime/helpers/classCallCheck');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var _classCallCheck__default = /*#__PURE__*/_interopDefaultLegacy(_classCallCheck);

var addListenerName = '-septima-listener-add-func';
var removeListenerName = '-septima-listener-remove-func';

function listen(target, listener) {
  var addListener = target[addListenerName];

  if (addListener) {
    addListener(listener);
    return function () {
      target[removeListenerName](listener);
    };
  } else {
    return null;
  }
}

function getPathData(anElement, aPath) {
  if (anElement && aPath) {
    var target = anElement;
    var path = aPath.split('.');
    var propName = path[0];

    for (var i = 1; i < path.length; i++) {
      target = target[propName];

      if (!target) {
        propName = null;
        break;
      } else propName = path[i];
    }

    if (propName) {
      return target[propName];
    } else return null;
  } else return null;
}

function setPathData(anElement, aPath, aValue) {
  if (anElement && aPath) {
    var target = anElement;
    var path = aPath.split('.');
    var propName = path[0];

    for (var i = 1; i < path.length; i++) {
      target = target[propName];

      if (!target) {
        propName = null;
      } else {
        propName = path[i];
      }
    }

    if (propName) {
      target[propName] = aValue;
    }
  }
}

function observeElements(target, propListener) {
  function subscribe(aData, aListener) {
    var nHandler = listen(aData, aListener);

    if (nHandler) {
      return nHandler;
    }

    return null;
  }

  var subscribed = [];
  target.forEach(function (item) {
    var remover = subscribe(item, propListener);

    if (remover) {
      subscribed.push(remover);
    }
  });
  return {
    unlisten: function unlisten() {
      subscribed.forEach(function (aEntry) {
        aEntry();
      });
    }
  };
}

function observePath(target, path, propListener) {
  function subscribe(aData, aListener, aPropName) {
    var listenReg = listen(aData, {
      change: function change(aChange) {
        if (!aPropName || aChange.propertyName === aPropName) {
          aListener.change(aChange);
        }
      }
    });

    if (listenReg) {
      return listenReg;
    } else {
      return null;
    }
  }

  var subscribed = [];
  var pathRebinder = {
    change: function change(evt) {
      subscribed.forEach(function (aEntry) {
        aEntry();
      });
      listenPath();
      var pathDatum = getPathData(target, path);
      propListener.change({
        source: target,
        propertyName: path,
        oldValue: pathDatum,
        newValue: pathDatum
      });
    }
  };

  function listenPath() {
    subscribed = [];
    var data = target;
    var pathItems = path.split('.');

    for (var i = 0; i < pathItems.length; i++) {
      var propName = pathItems[i];
      var listener = i === pathItems.length - 1 ? propListener : pathRebinder;
      var cookie = subscribe(data, listener, propName);

      if (cookie) {
        subscribed.push(cookie);
        if (data[propName]) data = data[propName];else break;
      } else {
        break;
      }
    }
  }

  if (target) {
    listenPath();
  }

  return {
    unlisten: function unlisten() {
      subscribed.forEach(function (removeSubscriber) {
        removeSubscriber();
      });
    }
  };
}

function Bound() {
  var self = this;
  var data = null;
  var path = null;
  var settingToWidget = false;

  function toWidget(datum) {
    if (!settingToData) {
      settingToWidget = true;

      try {
        self.value = datum;
      } finally {
        settingToWidget = false;
      }
    }
  }

  var settingToData = false;

  function toData(datum) {
    if (!settingToWidget) {
      settingToData = true;

      try {
        setPathData(data, path, datum);
      } finally {
        settingToData = false;
      }
    }
  }

  this.addValueChangeHandler(function (evt) {
    toData(evt.newValue);
  });
  var boundReg = null;

  function unbind() {
    if (boundReg) {
      boundReg.unlisten();
      boundReg = null;
    }
  }

  function bind() {
    if (data && path) {
      boundReg = observePath(data, path, {
        change: function change(evt) {
          toWidget(evt.newValue);
        }
      });
      toWidget(getPathData(data, path));
    } else {
      self.value = null;
    }
  }

  function rebind() {
    unbind();
    bind();
  }

  Object.defineProperty(this, 'data', {
    get: function get() {
      return data;
    },
    set: function set(aValue) {
      if (data !== aValue) {
        data = aValue;
        rebind();
      }
    }
  });
  Object.defineProperty(this, 'field', {
    get: function get() {
      return path;
    },
    set: function set(aValue) {
      if (path !== aValue) {
        path = aValue;
        rebind();
      }
    }
  });
}

var PathComparator = function PathComparator(path) {
  var ascending = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  var caseSensitive = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  _classCallCheck__default['default'](this, PathComparator);

  Object.defineProperty(this, 'ascending', {
    get: function get() {
      return ascending;
    }
  });

  function oCompare(od1, od2) {
    if (od1 == null && od2 == null) return 0;else if (od1 == null) return 1;else if (od2 == null) return -1;

    if (!caseSensitive) {
      if (typeof od1 === 'string') od1 = od1.toLowerCase();
      if (typeof od2 === 'string') od2 = od2.toLowerCase();
    }

    if (od1 === od2) return 0;else if (od1 > od2) return 1;else return -1;
  }

  function compare(o1, o2) {
    var oData1 = Bound.getPathData(o1, path);
    var oData2 = Bound.getPathData(o2, path);
    var res = oCompare(oData1, oData2);
    return ascending ? res : -res;
  }

  Object.defineProperty(this, 'compare', {
    get: function get() {
      return compare;
    }
  });
};

Object.defineProperty(Bound, 'PathComparator', {
  get: function get() {
    return PathComparator;
  }
});
Object.defineProperty(Bound, 'observeElements', {
  get: function get() {
    return observeElements;
  }
});
Object.defineProperty(Bound, 'listen', {
  get: function get() {
    return listen;
  }
});
Object.defineProperty(Bound, 'getPathData', {
  get: function get() {
    return getPathData;
  }
});
Object.defineProperty(Bound, 'setPathData', {
  get: function get() {
    return setPathData;
  }
});
Object.defineProperty(Bound, 'observePath', {
  get: function get() {
    return observePath;
  }
});

module.exports = Bound;

==> kenga-0.0.53.tgz: package/bound.js <==
'use strict';

var _classCallCheck = require('@babel/runtime/helpers/classCallCheck');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var _classCallCheck__default = /*#__PURE__*/_interopDefaultLegacy(_classCallCheck);

var addListenerName = '-septima-listener-add-func';
var removeListenerName = '-septima-listener-remove-func';

function listen(target, listener) {
  var addListener = target[addListenerName];

  if (addListener) {
    addListener(listener);
    return function () {
      target[removeListenerName](listener);
    };
  } else {
    return null;
  }
}

function getPathData(anElement, aPath) {
  if (anElement && aPath) {
    var target = anElement;
    var path = aPath.split('.');
    var propName = path[0];

    for (var i = 1; i < path.length; i++) {
      target = target[propName];

      if (!target) {
        propName = null;
        break;
      } else propName = path[i];
    }

    if (propName) {
      return target[propName];
    } else return null;
  } else return null;
}

function setPathData(anElement, aPath, aValue) {
  if (anElement && aPath) {
    var target = anElement;
    var path = aPath.split('.');
    var propName = path[0];

    for (var i = 1; i < path.length; i++) {
      target = target[propName];

      if (!target) {
        propName = null;
      } else {
        propName = path[i];
      }
    }

    if (propName) {
      target[propName] = aValue;
    }
  }
}

function observeElements(target, propListener) {
  function subscribe(aData, aListener) {
    var nHandler = listen(aData, aListener);

    if (nHandler) {
      return nHandler;
    }

    return null;
  }

  var subscribed = [];
  target.forEach(function (item) {
    var remover = subscribe(item, propListener);

    if (remover) {
      subscribed.push(remover);
    }
  });
  return {
    unlisten: function unlisten() {
      subscribed.forEach(function (aEntry) {
        aEntry();
      });
    }
  };
}

function observePath(target, path, propListener) {
  function subscribe(aData, aListener, aPropName) {
    var listenReg = listen(aData, {
      change: function change(aChange) {
        if (!aPropName || aChange.propertyName === aPropName) {
          aListener.change(aChange);
        }
      }
    });

    if (listenReg) {
      return listenReg;
    } else {
      return null;
    }
  }

  var subscribed = [];
  var pathRebinder = {
    change: function change(evt) {
      subscribed.forEach(function (aEntry) {
        aEntry();
      });
      listenPath();
      var pathDatum = getPathData(target, path);
      propListener.change({
        source: target,
        propertyName: path,
        oldValue: pathDatum,
        newValue: pathDatum
      });
    }
  };

  function listenPath() {
    subscribed = [];
    var data = target;
    var pathItems = path.split('.');

    for (var i = 0; i < pathItems.length; i++) {
      var propName = pathItems[i];
      var listener = i === pathItems.length - 1 ? propListener : pathRebinder;
      var cookie = subscribe(data, listener, propName);

      if (cookie) {
        subscribed.push(cookie);
        if (data[propName]) data = data[propName];else break;
      } else {
        break;
      }
    }
  }

  if (target) {
    listenPath();
  }

  return {
    unlisten: function unlisten() {
      subscribed.forEach(function (removeSubscriber) {
        removeSubscriber();
      });
    }
  };
}

function Bound() {
  var self = this;
  var data = null;
  var path = null;
  var settingToWidget = false;

  function toWidget(datum) {
    if (!settingToData) {
      settingToWidget = true;

      try {
        self.value = datum;
      } finally {
        settingToWidget = false;
      }
    }
  }

  var settingToData = false;

  function toData(datum) {
    if (!settingToWidget) {
      settingToData = true;

      try {
        setPathData(data, path, datum);
      } finally {
        settingToData = false;
      }
    }
  }

  this.addValueChangeHandler(function (evt) {
    toData(evt.newValue);
  });
  var boundReg = null;

  function unbind() {
    if (boundReg) {
      boundReg.unlisten();
      boundReg = null;
    }
  }

  function bind() {
    if (data && path) {
      boundReg = observePath(data, path, {
        change: function change(evt) {
          toWidget(evt.newValue);
        }
      });
      toWidget(getPathData(data, path));
    } else {
      self.value = null;
    }
  }

  function rebind() {
    unbind();
    bind();
  }

  Object.defineProperty(this, 'data', {
    get: function get() {
      return data;
    },
    set: function set(aValue) {
      if (data !== aValue) {
        data = aValue;
        rebind();
      }
    }
  });
  Object.defineProperty(this, 'field', {
    get: function get() {
      return path;
    },
    set: function set(aValue) {
      if (path !== aValue) {
        path = aValue;
        rebind();
      }
    }
  });
}

var PathComparator = function PathComparator(path) {
  var ascending = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  var caseSensitive = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  _classCallCheck__default["default"](this, PathComparator);

  Object.defineProperty(this, 'ascending', {
    get: function get() {
      return ascending;
    }
  });

  function oCompare(od1, od2) {
    if (od1 == null && od2 == null) return 0;else if (od1 == null) return 1;else if (od2 == null) return -1;

    if (!caseSensitive) {
      if (typeof od1 === 'string') od1 = od1.toLowerCase();
      if (typeof od2 === 'string') od2 = od2.toLowerCase();
    }

    if (od1 === od2) return 0;else if (od1 > od2) return 1;else return -1;
  }

  function compare(o1, o2) {
    var oData1 = Bound.getPathData(o1, path);
    var oData2 = Bound.getPathData(o2, path);
    var res = oCompare(oData1, oData2);
    return ascending ? res : -res;
  }

  Object.defineProperty(this, 'compare', {
    get: function get() {
      return compare;
    }
  });
};

Object.defineProperty(Bound, 'PathComparator', {
  get: function get() {
    return PathComparator;
  }
});
Object.defineProperty(Bound, 'observeElements', {
  get: function get() {
    return observeElements;
  }
});
Object.defineProperty(Bound, 'listen', {
  get: function get() {
    return listen;
  }
});
Object.defineProperty(Bound, 'getPathData', {
  get: function get() {
    return getPathData;
  }
});
Object.defineProperty(Bound, 'setPathData', {
  get: function get() {
    return setPathData;
  }
});
Object.defineProperty(Bound, 'observePath', {
  get: function get() {
    return observePath;
  }
});

module.exports = Bound;
